import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


class AnimationTicker extends TimerTask 
{
	SimDisplay disp;

	boolean running;
	int cframe=0;

	AnimationTicker(SimDisplay disp) 
	{
		this.disp = disp;
		running = false;
	}

	public void run() 
	{
		if (running) 
		{
			disp.kernel.stepTime(1);
			disp.updateStatusText();
			disp.repaint();
			cframe++;
		}
		else
			cframe=0;
	}
}

class SimDisplay extends JFrame implements ActionListener 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 5397605751167930551L;

	SimPanel sp;

	SimKernel kernel;

	JButton btnStep;

	JButton btnReset;

	JLabel lblStatus;

	JTextField tfEventNo;

	JButton btnGoToEvent;

	JButton btnGoToTime;

	JButton btnAnimate;
	
	JPanel panelToolBar;

	java.util.Timer timer;

	AnimationTicker ticker;

	public SimDisplay(SimKernel kernel) 
	{
		this.kernel = kernel;
		ticker = new AnimationTicker(this);
		timer = new java.util.Timer();
		timer.scheduleAtFixedRate(ticker, 0, 100);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				Frame frame = (Frame) evt.getSource();
				SimDisplay dsp = (SimDisplay) frame;
				dsp.timer.cancel();

				// Hide the frame
				frame.setVisible(false);

				// If the frame is no longer needed, call dispose
				frame.dispose();
			}
		});
		this.setBounds(0, 0, 800, 700);
		this.setLayout(null);
		sp = new SimPanel();
		sp.setBounds(0, 0, 800, 600);
		this.getContentPane().add(sp);
		
		panelToolBar=new JPanel();
		panelToolBar.setBounds(0,600,800,50);
		this.getContentPane().add(panelToolBar);

		btnStep = new JButton("Step");
		//btnStep.setBounds(10, 600, 75, 25);
		btnStep.addActionListener(this);
		panelToolBar.add(btnStep);

		btnReset = new JButton("Reset");
		//btnReset.setBounds(90, 600, 75, 25);
		btnReset.addActionListener(this);
		panelToolBar.add(btnReset);

		tfEventNo = new JTextField();
		tfEventNo.setMinimumSize(new Dimension(100,25));
		tfEventNo.setPreferredSize(new Dimension(100,25));
		//tfEventNo.setBounds(170, 600, 75, 25);
		panelToolBar.add(tfEventNo);

		btnGoToEvent = new JButton("Go to event");
		//btnGoToEvent.setBounds(250, 600, 75, 25);
		btnGoToEvent.addActionListener(this);
		panelToolBar.add(btnGoToEvent);

		btnGoToTime = new JButton("Go to Time");
		//btnGoToTime.setBounds(330, 600, 75, 25);
		btnGoToTime.addActionListener(this);
		panelToolBar.add(btnGoToTime);

		btnAnimate = new JButton("Animate");
		btnAnimate.setPreferredSize(new Dimension(90,25));
		//btnAnimate.setBounds(410, 600, 75, 25);
		btnAnimate.addActionListener(this);
		panelToolBar.add(btnAnimate);

		lblStatus = new JLabel("Initializing the simulation...");
		lblStatus.setBounds(10, 650, 800, 25);
		this.getContentPane().add(lblStatus);
	}

	public void actionPerformed(ActionEvent evt) 
	{
		if (evt.getSource() == btnStep) 
		{
			kernel.step();
		} 
		else if (evt.getSource() == btnReset) 
		{
			kernel.reset();
		} 
		else if (evt.getSource() == btnGoToEvent) 
		{
			kernel.goToEvent(Integer.parseInt(tfEventNo.getText())); // try
			// catch?
		} 
		else if (evt.getSource() == btnGoToTime) 
		{
			kernel.goToTime(Double.parseDouble(tfEventNo.getText()));
		}
		else if (evt.getSource() == btnAnimate) 
		{
			if (btnAnimate.getText() == "Animate") 
			{
				ticker.running = true;
				btnAnimate.setText("Stop");
				btnAnimate.repaint();
			} 
			else 
			{
				ticker.running = false;
				btnAnimate.setText("Animate");
				btnAnimate.repaint();
			}
		}
		updateStatusText();
		repaint();
	}

	public void repaint() 
	{
		super.repaint();
/*		Component[] cl = this.getComponents();
		for (int i = 0; i < cl.length; i++)
			cl[i].repaint();*/
	}
	public void update(Graphics g)
	{
		paint(g); 
	}

	public void updateStatusText() 
	{
		double dataRate=0;
		double bufferSize=0;
		for (int i=0;i<kernel.nodeTable.length;i++)
		{
			dataRate+=kernel.nodeTable[i].state;
			bufferSize+=kernel.nodeTable[i].bufferSize;
		}
		lblStatus.setText("Current Event:" + kernel.currentEvent
				+ " Current Time:" + kernel.getSecondsPassed()
				+ " Current Total Data Rate:"
				+ dataRate
				+ " Current Total Buffer Size:" + bufferSize);
	}

}