import java.util.*;


class SimKernel {
	static SimKernel theKernel;

	static int BStransmissionCost = 0;
	
	static boolean saveImages = false;
	
	static int nTrees=1; // number of trees 
	

	public EventRecord eventList[];
	
	public Statistics stats;

	/**
	 * This is a list of all nodes. It is addressed by node id. 
	 * I intentionally ignore the missing node ids. Let's leave this as future work.
	 */
	public SimTreeLeaf nodeTable[];
	
	/**
	 * This is the update order list to be used in communication  
	 * it is NOT ordered by any way 
	 */
	public SimTreeLeaf nodeUpdateOrder[];

	//public int nodeState[][];
/*	public SimTreeLeaf nodeTable[][];
	
	SimTreeLeaf nodeUpdateOrder[]; //the order in which nodes are updated 

	DistanceMatrix D;*/

	public int RowSize = 15;

	public int nRows;

	public long currentTime = 0; // current pctimestamp

	public int currentEvent = 0; // current event., this points to the first
	
	public long MessageDelay = 1000; // the required time for messaging in miliseconds
	
	public long lastMessageTime = 0; // when was the last message sent in infrastructure
	
	public double CommRange = 51; // Range of communication

	// unprocessed event

	public long initTime;
	
	Vector<SimTree> treeRoots;

//	public SimTreeLeaf treeRoot;

	public Salmon theFish;

	public Random rnd;

	public SimKernel(EventRecord[] eventList, double speed, long seed) {
		init(eventList, speed, 15, 4, seed);
	}

	public SimKernel(EventRecord[] eventList, double speed, int RowSize, int nRows,
			long seed)
	{
		init(eventList, speed, RowSize, nRows, seed);
	}

	void init(EventRecord[] eventList, double speed, int RowSize, int nRows,
			long seed) {
		if (seed > 0)
			rnd = new Random(seed);
		else
			rnd = new Random();
		theKernel = this;
		this.RowSize = RowSize;
		theFish = null; // reset stuff is complicated
		this.nRows = nRows;
		this.eventList = eventList;
		createNodes(nRows*RowSize); // WARNING THIS STILL ASSUMES A GRID
		distributeNodesToGrid();
		reset();
		treeRoots=new Vector<SimTree>();
		createTreeRoots(nTrees);

		// grow trees
		while (!allNodesConnected())
		{
			for (int i=0;i<treeRoots.size();i++)
				treeRoots.get(i).growTree();
		}
		boolean changeMade;
		do
		{
			changeMade=false;
			changeMade=changeMade || balanceClusters();
			changeMade=changeMade || shortenTrees();
			changeMade=changeMade || balanceTrees();
		}while(changeMade);
		// update trees
		for (int i=0;i<treeRoots.size();i++)
			treeRoots.get(i).update();
		
		theFish = new Salmon(this);
		Salmon.v = speed;
		theFish.reset();
		for (int i = 1; i < eventList.length; i++) {
			if (eventList[i].pctimestamp - eventList[i - 1].pctimestamp < 0) {
				System.err.println("out of order stamps");
			}
		}
		stats=new Statistics();
		// printTree();
	}

	/**
	 * Make all trees of similar size
	 *
	 */
	boolean balanceClusters()
	{
		boolean changeMade;
		boolean anyChangeMade=false;
		do
		{
			changeMade=false;
			for (int i=0;i<nodeTable.length;i++)
			{
				if (nodeTable[i].children.length==0) // if it is leaf
				{
					Vector<SimTreeLeaf> neigh=getNodesInRange(nodeTable[i]);
					for (int j=0;j<neigh.size();j++)
					{
						if (neigh.get(j).tree.nodeList.size()<nodeTable[i].tree.nodeList.size()-1)
						{
							nodeTable[i].parent.removeChild(nodeTable[i]);
							nodeTable[i].tree.nodeList.remove(nodeTable[i]);
							
							neigh.get(j).addChild(nodeTable[i]);
							nodeTable[i].tree.root.fixLevel();

							nodeTable[i].tree=neigh.get(j).tree;
							neigh.get(j).tree.nodeList.add(nodeTable[i]);
							neigh.get(j).tree.root.fixLevel();
							changeMade=true;
							anyChangeMade=true;
						}
					}
				}
			}
		}while (changeMade);
		return anyChangeMade;
	}
	/**
	 * Reduce the depth of trees by local modifications
	 *
	 */
	boolean shortenTrees()
	{
		boolean anyChangeMade=false;
		boolean changeMade;
		do
		{
			changeMade=false;
			for (int i=0;i<nodeTable.length;i++)
			{
				Vector<SimTreeLeaf> neigh=getNodesInRange(nodeTable[i]);
				for (int j=0;j<neigh.size();j++)
				{
					if (neigh.get(j).tree == nodeTable[i].tree && neigh.get(j).level<nodeTable[i].level-1)
					{
						nodeTable[i].parent.removeChild(nodeTable[i]);						
						neigh.get(j).addChild(nodeTable[i]);
						neigh.get(j).tree.root.fixLevel();
						changeMade=true;
						anyChangeMade=true;
					}
				}
			}
		}while (changeMade);
		return anyChangeMade;
	}
	
	boolean balanceTrees()
	{
		boolean anyChangeMade=false;
		boolean changeMade;
		do
		{
			changeMade=false;
			for (int i=0;i<treeRoots.size();i++)
			{
				if (treeRoots.get(i).root.children.length==1) 
				{
					SimTreeLeaf oldRoot=treeRoots.get(i).root;
					SimTreeLeaf newRoot=treeRoots.get(i).root.children[0];
					oldRoot.removeChild(newRoot);
					newRoot.setParent(null);
					newRoot.addChild(oldRoot);
					treeRoots.get(i).setRoot(newRoot);
					newRoot.fixLevel();
					changeMade=true;
					anyChangeMade=true;
				}
				else
				{
					SimTree tree=treeRoots.get(i);
					SimTreeLeaf maxDepthChild=tree.root.children[0];
					for (int j=1;j<tree.root.children.length;j++)
						if (tree.root.children[j].depth()>maxDepthChild.depth())
							maxDepthChild=tree.root.children[j];
					boolean swaprequired=true;
					for (int j=0;j<tree.root.children.length;j++)
						if (tree.root.children[j]!=maxDepthChild && tree.root.children[j].depth()>=maxDepthChild.depth()-1)
							swaprequired=false;
					if (swaprequired)
					{
						SimTreeLeaf oldRoot=tree.root;
						SimTreeLeaf newRoot=maxDepthChild;
						oldRoot.removeChild(newRoot);
						newRoot.setParent(null);
						newRoot.addChild(oldRoot);
						tree.setRoot(newRoot);
						newRoot.fixLevel();
						changeMade=true;
						anyChangeMade=true;						
					}
				}
			}
		}while(changeMade);
		return anyChangeMade;
	}

	
	void createNodes(int nNodes)
	{
		nodeTable=new SimTreeLeaf[nRows*RowSize];
		nodeUpdateOrder=new SimTreeLeaf[nRows*RowSize];
		for (int i=0;i<nNodes;i++)
		{
			nodeTable[i]=new SimTreeLeaf();
			nodeUpdateOrder[i]=nodeTable[i];
		}
	}
	
	void distributeNodesToGrid()
	{
		for (int i=0;i<nRows;i++)
			for (int j=0;j<RowSize;j++)
			{
				nodeTable[i*RowSize+j].pos.x=j*(CommRange-1)+45;
				nodeTable[i*RowSize+j].pos.y=i*(CommRange-1)+45;
			}
	}
	
	void createTreeRoots(int nRoots)
	{
		int idx;
		for (int i=0;i<nRoots;i++)
		{
			boolean weHaveRootThere;
			do
			{
				weHaveRootThere=false;
				idx=rnd.nextInt(nodeTable.length);
				for (int j=0;j<treeRoots.size();j++)
					if(treeRoots.get(j).root==nodeTable[idx])
						weHaveRootThere=true;
			}while(weHaveRootThere);
			SimTree t=new SimTree(nodeTable[idx]);
			treeRoots.add(t);
		}
	}

	Vector<SimTreeLeaf> getNodesInRange(SimTreeLeaf node)
	{
		Vector<SimTreeLeaf> v=new Vector<SimTreeLeaf>();
		for (int i=0;i<nodeTable.length;i++)
		{
			if (nodeTable[i]!=node && nodeTable[i].pos.distance(node.pos)<CommRange)
				v.add(nodeTable[i]);
		}
		return v;
		
	}
	
	boolean allNodesConnected()
	{
		for (int i=0;i<nodeTable.length;i++)
			if (nodeTable[i].isIsolated())
				return false;
		return true;
	}

	SimTreeLeaf getClosestNode(double x,double y)
	{
		SimTreeLeaf closest=nodeTable[0];
		for (int i=1;i<nodeTable.length;i++)
		{
			if (closest.pos.distance(x, y)>nodeTable[i].pos.distance(x,y))
				closest=nodeTable[i];
		}
		return closest;
	}



	void step() {
		step(true);
	}

	void step(boolean disp) // step at least one event
	{
		int cEvent = currentEvent;
		while (cEvent == currentEvent) {
			stepTime(1);
		}
		if (disp)
			System.out.println("Current Event: " + eventList[currentEvent]);
	}

	void stepEvent() // low level event increment. don't call this one use
	// stepTime
	{
		if (currentEvent == eventList.length)
			return; // don't step over it
		int nodeId = eventList[currentEvent].id;
		int type = eventList[currentEvent].type;
		currentTime = eventList[currentEvent].pctimestamp;
		nodeTable[nodeId].state= 1 - type; // 0 start 1 stop
		currentEvent++;
	}

	void stepTime(double seconds) // lowest level function for incementing
	// time
	{
		double deadline = (getSecondsPassed() + seconds) * 1000 + initTime;
		while (currentEvent < eventList.length
				&& eventList[currentEvent].pctimestamp < deadline) {
			stepEvent();
		} // do all the events
		currentTime = (long) deadline;
		theFish.step(seconds);
		if (currentTime-lastMessageTime > MessageDelay )
		{
			stepInfrastructure();
		}
		if (currentEvent >= eventList.length )
		{
			for (int j=0;j<nodeTable.length;j++)
				nodeTable[j].state=0; // reset states of nodes				
		}
		stats.stepStatistics();
	}
	
	void stepInfrastructure()
	{
		// here we randomize the order
		for (int i=0;i<nodeUpdateOrder.length*2;i++)
		{ // do random swaps
			SimTreeLeaf tmp;
			int a=rnd.nextInt(nodeUpdateOrder.length);
			int b=rnd.nextInt(nodeUpdateOrder.length); 
			tmp=nodeUpdateOrder[a];
			nodeUpdateOrder[a]=nodeUpdateOrder[b];
			nodeUpdateOrder[b]=tmp;
		}
		for (int i=0;i<nodeUpdateOrder.length;i++)
		{
			nodeUpdateOrder[i].step();
		}
		for (int i=0;i<nodeUpdateOrder.length;i++)
		{
			nodeUpdateOrder[i].update();
		}		
	}

	void goToTime(double seconds) 
	{
		goToTime(seconds, true);
	}

	void goToTime(double seconds, boolean disp) // in seconds
	{
		double deadline = seconds * 1000 + initTime;
		while (currentEvent < eventList.length && deadline < currentTime - 1000) 
		{
			stepTime(1);
		}
		stepTime((deadline - currentTime) / 1000);
	}

	void reset() 
	{
		currentTime = eventList[0].pctimestamp;
		initTime = currentTime;
		currentEvent = 0;
		if (theFish != null)
			theFish.reset();
		for (int i=0;i<nodeTable.length;i++)
			nodeTable[i].reset();
		lastMessageTime=0; 
	}

	void goToEvent(int n) 
	{
		goToEvent(n, true);
	}

	void goToEvent(int n, boolean disp) 
	{
		reset();
		while (currentEvent < n) 
		{
			step(disp);
		}
	}

	public double getSecondsPassed() 
	{
		return ((double) currentTime - initTime) / 1000;
	}

	/** Only returns the salmon cost for the tree that contains node n
	 * 
	 * @param n
	 * @return
	 */

	public double getCost(SimTreeLeaf n) 
	{
		double sum = 0;
		for (int i=0;i<treeRoots.size();i++)
		{
			SimTree t=treeRoots.get(i);
			if (t.contains(n))
			{
				for (int j = 0; j < t.nodeList.size(); j++) 
				{
					sum += t.D.getDistance(n, t.nodeList.get(j))* t.nodeList.get(j).state;
				}
			}
		}
		return sum;
	}
	


	public SimTreeLeaf getOptimalNode() 
	{
		return treeRoots.get(0).root.getOptimalNode();//////WARNING
	}


}