import java.util.*;
public class salmonSim 
{
	static SimDisplay sd; 

	public static void showHelp()
	{
		System.out.println("Salmon Emulator");
		System.out.println("USAGE:");
		System.out.println("salmonSim <filename> <options>");
		System.out.println("OPTIONS:");
		System.out.println("-----------------------------------");
		System.out.println("General Options");
		System.out.println("-nodisplay: Don't show the gui");
		System.out.println("-seed: the random number seed");
		System.out.println("-saveimages: save screen shot images from animation");
		System.out.println("-nruns <n>: number of runs. these are made with different seeds");
		System.out.println("-linkcapacity <n>: maximum amount of transmission between nodes [default 8]");
		System.out.println("-nRows <n>: number of rows");
		System.out.println("-nCols <n>: number of columns");
		System.out.println("-ntrees <n>: desired number of trees[default 1]");
		System.out.println("-timestrech <r>: the multiplier for timings of the events[default 1]");
		System.out.println();
		System.out.println("Data generation options");
		System.out.println("-sourceSpeed <r>: desired speed of distribution (unit is distance between adj. nodes)");
		System.out.println("-sourceVariance <r>: desired variance of distribution");
		System.out.println();
		System.out.println("Sensor node options");
		System.out.println("-bufferthreshold <n>: amount of buffering in the node [default 0]");
		System.out.println("-trackrobot: the arrow protocol tracks the robot for forwarding data");
		System.out.println();
		System.out.println("Mobile agent options");
		System.out.println("-superfastBS: super fast base station connection");
		System.out.println("-speed <r>: Speed of base station in terms of pixels (need calibration)");
		System.out.println("-useBScost: use BS transmission cost (d[i][j]++)");
		System.out.println("-treetourlength <n>: The length of the merhant tour on the tree in seconds ");
		System.out.println("                     Default is 0 for multiple trees");
		System.out.println("                     -1 (which means infinity) for single tree");
		System.out.println();
		System.out.println("Controller Options");
		System.out.println("-merchant: use merchant algorithm for decisions");
		System.out.println("-merchantv2: use merchant v2 which follows max buffer size");
		System.out.println("-merchantv3: use merchant v3 which follows weighted buffer size");
		System.out.println("-randomwalker: use a random walk, which is a little biased towards root");
		System.out.println("-simplefisher: use simple fisher controller");
		System.out.println("-tspfisher: use tsp fisher controller (only considers distance)");
		System.out.println("-mlpfisher: use mlp fisher controller (takes buffers into account)");
		System.out.println();
		System.out.println("Output options");
		System.out.println("-printxymatrix: print the xy matrix");
		System.out.println("-printcostchange: print the change in cost");
		System.out.println("-printtotalcosts: print the total costs");
		System.out.println("-printinstantchange: print the instaneous costs in each simulation step");
		System.out.println("-printBurstyTrace: print trace but reset at bursts");
		System.out.println("-printnodeusage: print the energy use of each node");
	}

	public static SimKernel realMain(String[] args, boolean showtext)
	{
		if (args.length < 1)
			showHelp();
		else
		{
			double speed = 20;
			long seed = -1;
			boolean hideGUI = false;
			boolean printxymatrix = false;
			boolean printcostchange = false;
			boolean printtotalcosts = false;
			boolean printinstantchange = false;
			boolean printBurstyTrace = false;
			boolean printnodeusage = false;
			int treeTourLength=-1;
			double timestrech=1;
			double sourceSpeed=0.1;
			double sourceVariance=0.15;
			SimKernel.saveImages=false;
			int nRows=10;
			int nCols=10;
			for (int i = 1; i < args.length; i++) 
			{
				if (args[i].equals("-speed")) 
					speed = Double.parseDouble(args[++i]);
				else if (args[i].equals("-seed")) 
					seed = Long.parseLong(args[++i]);
				else if (args[i].equals("-nodisplay"))
					hideGUI = true;
				else if (args[i].equals("-printxymatrix"))
					printxymatrix = true;
				else if (args[i].equals("-printcostchange"))
					printcostchange = true;
				else if (args[i].equals("-printtotalcosts"))
					printtotalcosts = true;
				else if (args[i].equals("-printinstantchange"))
					printinstantchange = true;
				else if (args[i].equals("-printBurstyTrace"))
					printBurstyTrace = true;
				else if (args[i].equals("-useBScost"))
					SimKernel.BStransmissionCost = 1;
				else if (args[i].equals("-printnodeusage"))
					printnodeusage=true;
				else if (args[i].equals("-saveimages"))
					SimKernel.saveImages=true;
				else if (args[i].equals("-nRows"))
					nRows=Integer.parseInt(args[++i]);
				else if (args[i].equals("-nCols"))
					nCols=Integer.parseInt(args[++i]);
				else if (args[i].equals("-superfastBS"))
					SimTreeLeaf.superFastBS=true;
				else if (args[i].equals("-merchant"))
					Salmon.useMerchant=true;
				else if (args[i].equals("-merchantv2"))
					Salmon.useMerchantv2=true;
				else if (args[i].equals("-merchantv3"))
					Salmon.useMerchantv3=true;
				else if (args[i].equals("-simplefisher"))
					Salmon.useSimpleFisher=true;
				else if (args[i].equals("-tspfisher"))
					Salmon.useTSPFisher=true;
				else if (args[i].equals("-mlpfisher"))
					Salmon.useMLPFisher=true;
				else if (args[i].equals("-randomwalker"))
					Salmon.useRandomWalker=true;
				else if (args[i].equals("-bufferthreshold"))
					SimTreeLeaf.bufferThreshold=Integer.parseInt(args[++i]);
				else if (args[i].equals("-linkcapacity"))
					SimTreeLeaf.LinkCapacity=Integer.parseInt(args[++i]);
				else if (args[i].equals("-ntrees"))
					SimKernel.nTrees=Integer.parseInt(args[++i]);
				else if (args[i].equals("-treetourlength"))
					treeTourLength=Integer.parseInt(args[++i]);
				else if (args[i].equals("-trackrobot"))
					SimTreeLeaf.trackrobot=true;
				else if (args[i].equals("-timestrech"))
					timestrech=Double.parseDouble(args[++i]);
				else if (args[i].equals("-sourceSpeed"))
					sourceSpeed=Double.parseDouble(args[++i]);
				else if (args[i].equals("-sourceVariance"))
					sourceVariance=Double.parseDouble(args[++i]);
				else
				{
					showHelp();
					System.out.println("unknown parameter: '"+args[i]+"'");
					System.exit(1);
				}
			}
			if (SimKernel.nTrees==1)
			{
				treeTourLength=-1;
			}
			Salmon.treeTourLength=treeTourLength;
			SimKernel krn;
			EventDB db=EventDB.createEventDB(args[0],nRows, nCols,timestrech ,seed, sourceSpeed, sourceVariance);
			krn = new SimKernel(db.getArray(), speed, db.nRows,db.RowSize, seed);
			Salmon.v = speed;
			if (printxymatrix) 
			{
				krn.stats.printXYMatrix();
			}
			else if (printcostchange) 
			{
				krn.stats.printCostTrace();
			}
			else if (printtotalcosts) 
			{
				System.out.println("" + 
						//krn.stats.getConstantTimeTotalCost() + 
						","
						+ krn.stats.getConstantTimeTotalCostMobile());
			}
			else if (printinstantchange) 
			{
				krn.stats.printInstantenousCostTrace();
			}
			else if (printBurstyTrace) 
			{
				krn.stats.printBurstyTrace();
			}
			else if (printnodeusage)
			{
				krn.stats.printNodeUsage(showtext);
			}
			else if (!hideGUI) 
			{
				sd = new SimDisplay(krn);
				sd.setVisible(true);
			}
			return krn;
		}
		return null;
	}

	public static void main(String[] args) 
	{
		boolean nrunsGiven=false;
		int nruns=1;
		for (int i=1;i<args.length;i++) // snoop arguments for multiple runs
		{
			if (args[i].equals("-nruns"))
			{
				nruns=Integer.parseInt(args[++i]);
				nrunsGiven=true;
			}
		}
		if (!nrunsGiven)
			realMain(args,true);
		else
		{
			Vector<String> args2=new Vector<String>();
			int seedArgIdx=-1;
			boolean printTotalCosts=false;
			for (int i=0;i<args.length;i++)
			{
				if (args[i].equals("-printtotalcosts"))
					printTotalCosts=true;
				if (args[i].equals("-seed"))
					seedArgIdx=args2.size()+1;
				if (args[i].equals("-nruns"))
					i++; // step over this and the next
				else
					args2.add(args[i]);
			}
			MultiRunStatistics mrs=new MultiRunStatistics(printTotalCosts);

			if (seedArgIdx==-1) // no seed argument is given so add these
			{
				args2.add("-seed");
				seedArgIdx=args2.size();
				args2.add("10236");
			}
			for (int i=0;i<nruns;i++)
			{
				args2.set(seedArgIdx, ""+(10236+i));
				String arglist[]=new String [args2.size()];
				for (int j=0;j<args2.size();j++)
					arglist[j]=args2.get(j);
				SimKernel krn=realMain(arglist, false);
				mrs.addStatistics(krn.stats);
			}
			mrs.printResult();
		}
	}

}